@@ -4,6 +4,7 @@ knitr::opts_chunk$set(
   comment = "#>"
 )
 
-## ----setup--------------------------------------------------------------------
-library(DR.SC)
+## ----eval=TRUE----------------------------------------------------------------
+
+library("DR.SC")
 
